@@ -2,10 +2,14 @@ package Outthentic::DSL;
 
 use strict;
 
-our $VERSION = '0.0.5';
+our $VERSION = '0.0.6';
 
 use Carp;
 use Data::Dumper;
+use Outthentic::DSL::Context::Range;
+use Outthentic::DSL::Context::Default;
+use Outthentic::DSL::Context::TextBlock;
+
 $Data::Dumper::Terse=1;
 
 sub results {
@@ -42,9 +46,11 @@ sub new {
 
     bless {
         results => [],
-        output_context => [],
-        search_context => [],
-        context_populated => 0,
+        original_context => [],
+        current_context => [],
+        context_modificator => Outthentic::DSL::Context::Default->new(),
+        has_context => 0,
+        succeeded => [],
         captures => [],
         within_mode => 0,
         block_mode => 0,
@@ -53,38 +59,44 @@ sub new {
         debug_mod => 0,
         output => $output||'',
         match_l => 40,
+        stream => {},
         %{$opts},
     }, __PACKAGE__;
 
 }
 
-sub populate_context {
+sub create_context {
 
     my $self = shift;
 
-    return if $self->{context_populated};
+    return if $self->{has_context};
 
     my $i = 0;
 
-    my @output_context = ();
+    my @original_context = ();
 
     for my $l ( split /\n/, $self->{output} ){
         chomp $l;
         $i++;
         $l=":blank_line" unless $l=~/\S/;
-        push @output_context, [$l, $i];
+        push @original_context, [$l, $i];
+
+        $self->add_debug_result("[oc] [$l, $i]") if $self->{debug_mod} >= 2;
+
     }
 
-    $self->{output_context} = [@output_context];
-    $self->{search_context} = [@output_context];
+    $self->{original_context} = [@original_context];
+    $self->{current_context} = [@original_context];
 
     $self->add_debug_result('context populated') if $self->{debug_mod} >= 2;
 
-    $self->{context_populated} = 1;
+
+    $self->{has_context} = 1;
 
 
 }
 
+
 sub reset_captures {
 
     my $self = shift;
@@ -92,6 +104,49 @@ sub reset_captures {
 
 }
 
+sub reset_succeeded {
+
+    my $self = shift;
+    $self->{succeeded} = [];
+
+}
+
+sub reset_context {
+
+    my $self = shift;
+
+    $self->{current_context} = $self->{original_context};
+
+    $self->add_debug_result('reset search context') if $self->{debug_mod} >= 2;
+
+    $self->{context_modificator} = Outthentic::DSL::Context::Default->new();
+
+}
+
+sub stream {
+
+    my $self = shift;
+    my @stream;
+    my $i=0;
+
+    for my $cid ( sort { $a <=> $b } keys  %{$self->{stream}} ){
+        $stream[$i]=[];
+        for my $c (@{$self->{stream}->{$cid}}){
+            push @{$stream[$i]}, $c->[0];
+            $self->add_debug_result("[stream {$cid} [$i]] $c->[0]") if $self->{debug_mod} >= 2;
+        }
+        $i++;
+    }
+    [@stream]
+}
+
+sub match_lines {
+
+    my $self = shift;
+    return $self->{succeeded};
+}
+
+
 sub check_line {
 
     my $self = shift;
@@ -106,37 +161,66 @@ sub check_line {
 
     my @captures = ();
 
-    $self->populate_context;
+    $self->create_context;
 
-    $self->add_debug_result("lookup $pattern ...") if $self->{debug_mod} >= 2;
+    $self->add_debug_result("[lookup] $pattern ...") if $self->{debug_mod} >= 2;
 
-    my @output_context         = @{$self->{output_context}};
-    my @search_context   = @{$self->{search_context}};
-    my @context_new     = ();
+    my @original_context   = @{$self->{original_context}};
+    my @context_new        = ();
+
+    # dynamic context 
+    my $dc = $self->{context_modificator}->change_context(
+        $self->{current_context},
+        $self->{original_context},
+        $self->{succeeded}
+    );
+
+    $self->add_debug_result("context modificator applied: ".(ref $self->{context_modificator})) 
+        if $self->{debug_mod} >=1;
+        
+    if ( $self->{debug_mod} >= 2 ) {
+        for my $dcl (@$dc){ 
+            $self->add_debug_result("[dc] $dcl->[0]");
+        } 
+
+    };
+    
+
+    $self->reset_succeeded;
 
     if ($check_type eq 'default'){
-        for my $c (@search_context){
-            my $ln = $c->[0]; my $next_i = $c->[1];
+        for my $c (@{$dc}){
+
+            my $ln = $c->[0];
+            next if $ln =~/#dsl_note:/;
+
             if ( index($ln,$pattern) != -1){
                 $status = 1;
                 $self->{last_match_line} = $ln;
-                push @context_new, $output_context[$next_i] if $self->{block_mode};
+                push @{$self->{succeeded}}, $c;
             }
         }
 
     }elsif($check_type eq 'regexp'){
-        for my $c (@search_context){
+
+
+        for my $c (@{$dc}) {
+
             my $re = qr/$pattern/;
-            my $ln = $c->[0]; my $next_i = $c->[1];
+
+            my $ln = $c->[0];
+
+            next if $ln eq ":blank_line";
+            next if $ln =~/#dsl_note:/;
 
             my @foo = ($ln =~ /$re/g);
 
             if (scalar @foo){
                 push @captures, [@foo];
                 $status = 1;
+                push @{$self->{succeeded}}, $c;
                 push @context_new, $c if $self->{within_mode};
                 $self->{last_match_line} = $ln;
-                push @context_new, $output_context[$next_i] if $self->{block_mode};
             }
 
         }
@@ -149,29 +233,44 @@ sub check_line {
     $self->{last_check_status} = $status;
 
     if ( $self->{debug_mod} >= 2 ){
-        $self->add_debug_result('captures:');
-        $self->add_debug_result(Dumper(\@captures));
+
+        my $i = -1;
+        my $j = -1;
+        for my $cpp (@captures){
+            $i++;
+            for my $cp (@{$cpp}){
+                $j++;
+                $self->add_debug_result("CAP[$i,$j]: $cp");
+            }
+            $j=0;
+        }
+
+        for my $s (@{$self->{succeeded}}){
+            $self->add_debug_result("SUCC: $s->[0]");
+        }
     }
 
     $self->{captures} = [ @captures ];
 
     # update context
-    if ( $self->{block_mode} and $status ){
-        $self->{search_context} = [@context_new];
-        $self->add_debug_result('block mode: modify search context to: '.(Dumper([@context_new]))) if $self->{debug_mod} >= 2
-    }elsif ( $self->{block_mode} and ! $status ){
-        $self->{search_context} = [];
-        $self->add_debug_result('block mode: modify search context to: '.(Dumper([@context_new]))) if $self->{debug_mod} >= 2
-    }elsif ( $self->{within_mode} and $status ){
-        $self->{search_context} = [@context_new];
+    if ( $self->{within_mode} and $status ){
+        $self->{current_context} = [@context_new];
         $self->add_debug_result('within mode: modify search context to: '.(Dumper([@context_new]))) if $self->{debug_mod} >= 2 
     }elsif ( $self->{within_mode} and ! $status ){
-        $self->{search_context} = []; # empty context if within expression has not passed 
+        $self->{current_context} = []; # empty context if within expression has not passed 
         $self->add_debug_result('within mode: modify search context to: '.(Dumper([@context_new]))) if $self->{debug_mod} >= 2 
     }
 
     $self->add_result({ status => $status , message => $message });
 
+
+    $self->{context_modificator}->update_stream(
+        $self->{current_context},
+        $self->{original_context},
+        $self->{succeeded}, 
+        \($self->{stream}),
+    );
+
     return $status;
 
 }
@@ -202,29 +301,57 @@ sub validate {
 
         chomp $l;
 
-        $self->add_debug_result($l) if $self->{debug_mod} >= 3;
+        $self->add_debug_result("[dsl] $l") if $self->{debug_mod} >= 2;
 
         next LINE unless $l =~ /\S/; # skip blank lines
 
         next LINE if $l=~ /^\s*#(.*)/; # skip comments
 
         if ($l=~ /^\s*begin:\s*$/) { # begin of text block
+            confess "you can't switch to text block mode when within mode is enabled" 
+                if $self->{within_mode};
+
+            $self->{context_modificator} = Outthentic::DSL::Context::TextBlock->new();
+
             $self->add_debug_result('begin text block') if $self->{debug_mod} >= 2;
             $self->{block_mode} = 1;
+
+            $self->reset_succeeded;
+
             next LINE;
         }
+
         if ($l=~ /^\s*end:\s*$/) { # end of text block
 
             $self->{block_mode} = 0;
 
-            # restore local context
-            $self->{search_context} = $self->{output_context};
+            $self->reset_context;
 
             $self->add_debug_result('end text block') if $self->{debug_mod} >= 2;
 
             next LINE;
         }
 
+        if ($l=~ /^\s*reset_context:\s*$/) {
+
+            $self->reset_context;
+
+            next LINE;
+        }
+
+        if ($l=~ /^\s*between:\s+(.*)/) { # set new context
+            
+            $self->{context_modificator} = Outthentic::DSL::Context::Range->new($1);
+
+            confess "you can't set context modificator when within mode is enabled" 
+                if $self->{within_mode};
+
+            confess "you can't set context modificator when text block mode is enabled" 
+                if $self->{block_mode};
+
+            next LINE;
+        }
+
         # validate unterminated multiline chunks
         if ($l=~/^\s*(regexp|code|generator|within|validator):\s*.*/){
             confess "unterminated multiline $chunk_type found, last line: $multiline_chunk[-1]" if defined($chunk_type);
@@ -275,6 +402,9 @@ sub validate {
 
         }elsif($l=~/^\s*within:\s*(.*)/){
 
+            confess "you can't switch to within mode when text block mode is enabled" 
+                if $self->{block_mode};
+
             my $re=$1;
             $self->handle_within($re);
 
@@ -339,14 +469,14 @@ sub handle_validator {
         confess "eval error; sub:handle_validator; code:$code; error: $@" if $@;
         confess "not valid return from validator, should be ARRAYREF. got: @{[ref $r]}" unless ref($r) eq 'ARRAY' ;
         $self->add_result({ status => $r->[0] , message => $r->[1] });
-        $self->add_debug_result("handle_validator OK. $code") if $self->{debug_mod} >= 3;
+        $self->add_debug_result("handle_validator OK (status: $r->[0] message: $r->[1]). $code") if $self->{debug_mod} >= 2;
     } else {
         my $code_to_eval = join "\n", @$code;
         my $r = eval "package main; $code_to_eval";
         confess "eval error; sub:handle_validator; code:$code_to_eval; error: $@" if $@;
         confess "not valid return from validator, should be ARRAYREF. got: @{[ref $r]}" unless ref($r) eq 'ARRAY' ;
         $self->add_result({ status => $r->[0] , message => $r->[1] });
-        $self->add_debug_result("handle_validator OK. multiline. $code_to_eval") if $self->{debug_mod} >= 3;
+        $self->add_debug_result("handle_validator OK. multiline. $code_to_eval") if $self->{debug_mod} >= 2;
     }
 
 }
@@ -380,12 +510,12 @@ sub handle_regexp {
     
     my $m;
 
-    my $reset_search_context = 0;
+    my $reset_context = 0;
 
     if ($self->{within_mode}){
 
         $self->{within_mode} = 0; 
-        $reset_search_context = 1;
+        $reset_context = 1;
 
         if ($self->{last_check_status}){
             my $lml =  $self->_short_string($self->{last_match_line});
@@ -401,7 +531,7 @@ sub handle_regexp {
 
     $self->check_line($re, 'regexp', $m);
 
-    $self->{search_context} = $self->{output_context} if $reset_search_context; 
+    $self->reset_context if $reset_context; 
 
     $self->add_debug_result("handle_regexp OK. $re") if $self->{debug_mod} >= 3;
 
@@ -441,12 +571,12 @@ sub handle_plain {
 
     my $m;
     my $lshort =  $self->_short_string($l);
-    my $reset_search_context = 0;
+    my $reset_context = 0;
 
     if ($self->{within_mode}){
         
         $self->{within_mode} = 0;
-        $reset_search_context = 1;
+        $reset_context = 1;
 
         if ($self->{last_check_status}){
             my $lml =  $self->_short_string($self->{last_match_line});
@@ -462,7 +592,7 @@ sub handle_plain {
 
     $self->check_line($l, 'default', $m);
 
-    $self->{search_context} = $self->{output_context} if $reset_search_context; 
+    $self->reset_context if $reset_context; 
 
     $self->add_debug_result("handle_plain OK. $l") if $self->{debug_mod} >= 3;
 }
@@ -485,6 +615,9 @@ sub _short_string {
 
 __END__
 
+=pod
+
+
 =encoding utf8
 
 
@@ -495,7 +628,13 @@ Language to validate text output.
 
 =head1 Install
 
-    cpanm Outthentic
+    cpanm Outthentic::DSL
+
+
+=head1 Informal introduction
+
+Alternative introduction into outthentic dsl in more in infromal way could be found here 
+- L<intro.md|https://github.com/melezhik/outthentic-dsl/blob/master/intro.md>
 
 
 =head1 Glossary
@@ -536,6 +675,17 @@ It's convenient to refer to the text validate by as stdout, thinking that one pr
 which is then validated.
 
 
+=head2 Search context
+
+A synonym for stdout term with emphasis of the fact that validation process if carried out in a given context.
+
+But default search context is equal to original stdout stream. 
+
+Parser verifies all stdout against a list of check expressions. 
+
+But see L<search context modificators|#search-context-modificators> section.
+
+
 =head2 Parser
 
 Parser is the program which:
@@ -574,7 +724,7 @@ generating new outthentic entries if entry is generator one
 
 =item *
 
-execution of perl code if entry is perl expression one
+execution of Perl code if entry is Perl expression one
 
 
 
@@ -620,7 +770,7 @@ generating validation results could be retrieved later
 
 =item *
 
-a final I<presentation> of validation results should be implimeted in a certain L<client|#clients> I<using> L<parser api|#parser-api> and not being defined at DSL scope. For the sake of readability a table like form ( which is a fake one ) is used for validation results in this document. 
+a final I<presentation> of validation results should be implemented in a certain L<client|#clients> I<using> L<parser api|#parser-api> and not being defined at DSL scope. For the sake of readability a table like form ( which is a fake one ) is used for validation results in this document. 
 
 
 
@@ -772,22 +922,27 @@ Check expressions:
 
 =item *
 
-plain strings
+plain     strings
 
 
 =item *
 
-regular expressions
+regular   expressions
 
 
 =item *
 
-text blocks
+text      blocks
 
 
 =item *
 
-within expressions
+within    expressions
+
+
+=item *
+
+range     expressions
 
 
 =item *
@@ -900,7 +1055,7 @@ Comment lines start with `#' symbol, comments chunks are ignored by parser:
 
     # comments could be represented at a distinct line, like here
     The beginning of story
-    Hello World # or could be added to existed expression to the right, like here
+    Hello World # or could be added for the existed expression to the right, like here
 
 
 =head2 Blank lines
@@ -979,20 +1134,20 @@ Parser will remain in a `text block' mode till the end of check file, which is p
 
 =head1 Perl expressions
 
-Perl expressions are just a pieces of perl code to I<get evaled> during parsing process. This is how it works:
+Perl expressions are just a pieces of Perl code to I<get evaled> during parsing process. This is how it works:
 
-    # perl expression between two check expressions
+    # Perl expression between two check expressions
     Once upon a time
     code: print "hello I am Outthentic"
     Lived a boy called Outthentic
 
-Internally once check file gets parsed this piece of DSL code is "turned" into regular perl code:
+Internally once check file gets parsed this piece of DSL code is "turned" into regular Perl code:
 
     execute_check_expression("Once upon a time");
     eval 'print "Lived a boy called Outthentic"';
     execute_check_expression("Lived a boy called Outthentic");
 
-One of the use case for perl expressions is to store L<`captures'|#captures> data:
+One of the use case for Perl expressions is to store L<`captures'|#captures> data:
 
     regexp: my name is (\w+) and my age is (\d+)
     code: $main::data{name} = capture()->[0]; $main::data{age} = capture()->[1]; 
@@ -1001,13 +1156,13 @@ One of the use case for perl expressions is to store L<`captures'|#captures> dat
 
 =item *
 
-Perl expressions are executed by perl eval function in context of C<package main>, please be aware of that.
+Perl expressions are executed by Perl eval function in context of C<package main>, please be aware of that.
 
 
 
 =item *
 
-Follow L<http://perldoc.perl.org/functions/eval.html|http://perldoc.perl.org/functions/eval.html> to get know more about perl eval.
+Follow L<http://perldoc.perl.org/functions/eval.html|http://perldoc.perl.org/functions/eval.html> to get know more about Perl eval.
 
 
 
@@ -1020,7 +1175,7 @@ Follow L<http://perldoc.perl.org/functions/eval.html|http://perldoc.perl.org/fun
 
 =item *
 
-Validator expressions like perl expressions are just a piece of perl code. 
+Validator expressions like Perl expressions are just a piece of Perl code. 
 
 
 
@@ -1067,7 +1222,7 @@ This is a simple example:
     # stdout
     # it's my family ages.
     alex    38
-    julia   25
+    julia   32
     jan     2
     
     
@@ -1094,7 +1249,7 @@ Generators is the way to I<generate new outthentic entries on the fly>.
 
 =item *
 
-Generator expressions like perl expressions are just a piece of perl code.
+Generator expressions like Perl expressions are just a piece of Perl code.
 
 
 
@@ -1207,7 +1362,7 @@ For example:
 
 Use text blocks if you want to achieve multiline checks.
 
-However when writing perl expressions, validators or generators one could use multilines strings.
+However when writing Perl expressions, validators or generators one could use multilines strings.
 
 `\' delimiters breaks a single line text on a multi lines:
 
@@ -1232,7 +1387,7 @@ Captures are pieces of data get captured when parser validates stdout against a
     # stdout
     # it's my family ages.
     alex    38
-    julia   25
+    julia   32
     jan     2
     
     
@@ -1311,24 +1466,60 @@ You also may use `capture()' function to get a I<first element> of captures arra
     validator: [ ( capture()->[0] >  10 ), " first number is greater than 10 " ];
 
 
-=head1 Within expressions
+=head1 Search context modificators
 
-Within expression acts like regular expression but narrows search context to last matching line:
+Search context modificators are special check expressions which not only validate stdout but modify search context.
 
-    # one of 3 colors:
-    within: color: (red|green|blue)
+But default search context is equal to original stdout. 
+
+That means outthentic parser execute validation process against original stdout stream
+
+There are two search context modificators to change this behavior:
+
+=over
+
+=item *
+
+within expressions
+
+
+
+=item *
+
+range expressions
+
+
+
+=back
+
+
+=head2 Within expressions
+
+Within expression acts like regular expression - parser checks stdout against given pattern 
+
+    # stdout
     
-    # if within expression is successfully passed
-    # new search context is last matching line  
-
-In other words when `:within' marker is used parser tries to validate stdout against regular expression following after :within marker and 
-if validation is successful new search context is defined:
-
-    # one of 3 colors:
-    within: color: (red|green|blue)
+    These are my colors
     
-    # I really need a red color
-    red
+    color: red
+    color: green
+    color: blue
+    color: brown
+    color: back
+    
+    That is it!
+    
+    # outthentic check
+    
+    # I need one of 3 colors:
+    
+    within: color: (red|green|blue)
+
+Then if checks given by within statement succeed I<next> checks will be executed I<in a context of>
+succeeded lines:
+
+    # but I really need a green one
+    green
 
 The code above does follows:
 
@@ -1336,21 +1527,42 @@ The code above does follows:
 
 =item *
 
-try to find `color:' followed by `red' or `green' or `blue' word 
+try to validate stdout against regular expression "color: (red|green|blue)"
+
 
 
 =item *
 
-if previous check is successful new context is narrowed to matching line
+if validation is successful new search context is set to all I<matching> lines
 
 
-=item *
-
-thus next plain string checks expression means - try to find `red' in line matching the `color: (red|green|blue)'
-
 
 =back
 
+These are:
+
+    color: red
+    color: green
+    color: blue
+
+=over
+
+=item *
+
+thus next plain string checks expression will be executed against new search context
+
+
+=back
+
+The result will be:
+
+    +--------+------------------------------------------------+
+    | status | message                                        |
+    +--------+------------------------------------------------+
+    | OK     | matches /color: (red|green|blue)/              |
+    | OK     | /color: (red|green|blue)/ matches green        |
+    +--------+------------------------------------------------+
+
 Here more examples:
 
     # try to find a date string in following format
@@ -1370,6 +1582,344 @@ Within expressions could be sequential, which effectively means using `&&' logic
     # and try to find month 04 in a date string
     within: \d\d\d\d-04-\d\d
 
+Speaking in human language chained within expressions acts like specifications. When you may start with some
+generic assumptions and then make your requirements more specific. A failure on any step of chain results in
+immediate break. 
+
+
+=head1 Range expressions
+
+Between or range expressions also act like I<search context modificators> - they change search area to one included
+I<between> lines matching right and left regular expression of between statement.
+
+It is very similar to what Perl L<range operator|http://perldoc.perl.org/perlop.html#Range-Operators> does 
+when extracting pieces of lines inside stream
+
+    while (<STDOUT>){
+        if /foo/ ... /bar/
+    }
+
+Outthentic analogy for this is between expression:
+
+    between: foo bar
+
+Between expression takes 2 arguments - left and right regular expression to setup search area boundaries.
+
+A search context will be all the lines included between line matching left expression and line matching right expression.
+
+A matching (boundary) lines are not included in range:
+
+These are few examples:
+
+Parsing html output
+
+    # stdout
+    <table cols=10 rows=10>
+        <tr>
+            <td>one</td>
+        </tr>
+        <tr>
+            <td>two</td>
+        </tr>
+        <tr>
+            <td>the</td>
+        </tr>
+    </table>
+    
+    
+    # between expression:
+    between: <table.*> <\/table>
+    regexp: <td>(\S+)<\/td>
+    
+    # or even so
+    between: <tr.*> <\/tr>
+    regexp: <td>(\S+)<\/td>
+
+Between expressions could not be nested, every new between expression discards old search context
+and setup new one:
+
+    # sample stdout
+    
+    foo
+    
+        1
+        2
+        3
+    
+        FOO
+            100
+        BAR
+    
+    bar
+    
+    FOO
+    
+        10
+        20
+        30
+    
+    BAR
+    
+    # outthentic check list:
+    
+    between: foo bar
+    
+    code: print "# foo/bar start"
+    
+    # here will be everything
+    # between foo and bar lines
+    
+    regexp: \d+
+    
+    code:                           \
+    for my $i (@{captures()}) {     \
+        print "# ", $i->[0], "\n"   \
+    }                               \
+    print "foo/bar end"
+    
+    between: FOO BAR
+    
+    code: print "# FOO/BAR start"
+    
+    # here will be everything
+    # between FOO and BAR lines
+    # NOT necessarily inside foo bar block
+    
+    regexp: \d+
+    
+    code:                           \
+    for my $i (@{captures()}) {     \
+        print "#", $i->[0], "\n";   \
+    }                               \
+    print "# FOO/BAR end"
+    
+    # TAP output
+    
+        # foo/bar start
+        # 1
+        # 2
+        # 3
+        # 100
+        # foo/bar end
+    
+        # FOO/BAR start
+        # 100
+        # 10
+        # 20
+        # 30
+        # FOO/BAR end
+
+And finally to restore search context use `reset_context:' statement:
+
+    # stdout
+    
+    hello
+    foo
+        hello
+        hello
+    bar
+    
+    
+    between foo bar
+    
+    # all check expressions here
+    # will be applied to the chunks
+    # between /foo/ ... /bar/
+    
+    hello       # should match 2 times
+    
+    # if you want to get back to an original search context
+    # just say reset_context:
+    
+    reset_context:
+    hello       # should match three times
+
+Range expressions caveats
+
+=over
+
+=item *
+
+range expressions don't keep original order inside range
+
+
+=back
+
+For example:
+
+    # stdout
+    
+    foo
+        1
+        2
+        1
+        2
+    bar
+    
+    
+    # outthentic check
+    
+    between: foo bar
+        regexp: 1
+        code: print '#', ( join ' ', map {$_->[0]} @{captures()} ), "\n"
+        regexp: 2
+        code: print '#', ( join ' ', map {$_->[0]} @{captures()} ), "\n"
+    
+    # validation output
+    
+        # 1 1
+        # 2 2
+
+=over
+
+=item *
+
+if you need precise order keep preserved use text blocks
+
+
+=back
+
+
+=head1 Experimental features
+
+Below is highly experimental features purely tested. You may use it on your own risk! ;)
+
+
+=head2 Streams
+
+Streams are alternative for captures. Consider following example:
+
+    # stdout
+    
+    foo
+        a
+        b
+        c
+    bar
+    
+    foo
+        1
+        2
+        3
+    bar
+    
+    foo
+        0
+        00
+        000
+    bar
+    
+    # outthentic check list
+    
+    begin:
+    
+        foo
+    
+            regexp: (\S+)
+            code: print '#', ( join ' ', map {$_->[0]} @{captures()} ), "\n"
+    
+            regexp: (\S+)
+            code: print '#', ( join ' ', map {$_->[0]} @{captures()} ), "\n"
+    
+            regexp: (\S+)
+            code: print '#', ( join ' ', map {$_->[0]} @{captures()} ), "\n"
+    
+    
+        bar
+    
+    end:
+
+The code above will print:
+
+    # a 1 0
+    # b 2 00
+    # c 3 000
+
+Notice something interesting? Output direction has been inverted.
+
+The reason for this is outthentic check expression works in "line by line scanning" mode 
+when output gets verified line by line against given check expression. Once all lines are matched
+they get dropped into one heap without preserving original "group context". 
+
+What if we would like to print all matching lines grouped by text blocks they belong to which is more convenient?
+
+This is where streams feature comes to rescue.
+
+Streams - are all the data successfully matched for given I<group context>. 
+
+Streams are available for text blocks and range expressions.
+
+Let's rewrite the example:
+
+    begin:
+    
+        foo
+            regexp: \S+
+            regexp: \S+
+            regexp: \S+
+        bar
+    
+        code:                                   \
+            for my $s (@{stream()}) {           \
+                print "# ";                     \
+                for my $i (@{$s}){              \
+                    print $i;                   \
+                }                               \
+                print "\n";                     \
+            }
+        
+    end:
+
+Stream function returns an arrays of streams. Every stream holds all the matched lines for given block.
+So streams preserve group context. Number of streams relates to the number of successfully matched blocks.
+
+Streams data presentation is much closer to what was originally given in stdout:
+
+    # foo a b  c    bar
+    # foo 1 2  3    bar
+    # foo 0 00 000  bar
+
+Stream could be specially useful when combined with range expressions where sizes
+of successfully matched blocks could be different:
+
+    # stdout
+    
+    foo
+        2
+        4
+        6
+        8
+    bar
+    
+    foo
+        1
+        3
+    bar
+    
+    
+    # outthentic check
+    
+    
+    between: foo bar
+    
+    regexp: \d+
+    
+    code:                                   \
+        for my $s (@{stream()}) {           \
+            print "# ";                     \
+            for my $i (@{$s}){              \
+                print $i;                   \
+            }                               \
+            print "\n";                     \
+        }
+    
+    
+    # validation output:
+    
+    
+    # 2 4 6 8
+    # 1 3
+
 
 =head1 Author
 
